! NOTE: This program was incorporated into rscalc_interp_acc.for.
! Note that rdrvaa includes initial conditions in the argument
! list, while rscalc_interp_acc does not.

!----------------- BEGIN RDRVAA -----------------------------
      subroutine rdrvaa(acc,na,omega,damp,dt,rd,rv,aa, d0, v0)
! This is a modified version of "Quake.For", originally
! written by J.M. Roesset in 1971 and modified by
! Stavros A. Anagnostopoulos, Oct. 1986.  The formulation is that of
! Nigam and Jennings (BSSA, v. 59, 909-922, 1969).  

!   acc = acceleration time series
!    na = length of time series
! omega = 2*pi/per
!  damp = fractional damping (e.g., 0.05)
!    dt = time spacing of input
!    rd = relative displacement of oscillator
!    rv = relative velocity of oscillator
!    aa = absolute acceleration of oscillator
! d0,v0 = initial displacement and velocity (usually set to 0.0)

! Dates: 02/11/00 - Modified by David M. Boore, based on RD_CALC
!        03/11/01 - Double precision version
!        03/14/01 - Added d0, v0 (note on 05 March 2010: I recommend 
!                   that they not be used, by setting them to 0.0.  
!                   I've kept them as arguments in the subroutine call
!                   so that I do not have to modify programs that use
!                   this subroutine).                   
!        03/14/01 - Changed name back to rdrvaa
!        01/31/03 - Moved implicit statement before the type declarations
!        10/10/07 - Initial variable assignments and iteration loop modified 
!                   to double-precision (Chris Stephens)
!        03/05/10 - Delete old (single precision) lines of code
!        12/22/10 - Remove minus sign in front of the initialization of y, ydot. 
!                   The minus sign was a remnant of an earlier version where I did not
!                   understand the meaning of y and ydot.
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)
!        05/16/15 - Use Implicit None and replace real*4 and double precision with modern equivalents

      real(4) :: acc(*), omega, damp, dt, rd, rv, aa, d0, v0
      
      real(8) :: d2, bom, d3, omd, om2, c1, omt, omdt, c2, c3, c4, ss, 
     :           cc, bomt, ee, s1, s2, s3, a11, a12, a21, a22,
     :           s4, s5, b11, b12, b21, b22, y, ydot, y1, z, z1, z2, ra
     
      integer :: i, na
      
      
      d2=1.d0-dble(damp)*dble(damp)
      d2=dsqrt(d2)
      bom=dble(damp)*dble(omega)
      d3 = 2.d0*bom                 ! for aa
      omd=dble(omega)*d2
      om2=dble(omega)*dble(omega)
      c1=1.d0/om2
      
      omt=dble(omega)*dble(dt)
      omdt=omd*dble(dt)
      c2=2.d0*dble(damp)/(om2*omt)
      c3=c1+c2
      c4=1.d0/(dble(omega)*omt)
      ss=dsin(omdt)
      cc=dcos(omdt)
      bomt=dble(damp)*omt
      ee=dexp(-bomt)
      ss=ss*ee
      cc=cc*ee
      s1=ss/omd
      s2=s1*bom
      s3=s2+cc
      a11=s3
      a12=s1
      a21=-om2*s1
      a22=cc-s2
      s4=c4*(1.d0-s3)
      s5=s1*c4+c2
      b11=s3*c3-s5
      b12=-c2*s3+s5-c1
      b21=-s1+s4
      b22=-s4
      
      rd=0.
      rv = 0.                           ! for rv
      aa = 0.                           ! for aa
      
      y=    dble(d0)
      ydot= dble(v0)    
!      y=0.
!      ydot=0.

      do i=1, na-1
      
        y1=a11*y+a12*ydot+b11*dble(acc(i))+b12*dble(acc(i+1))
        ydot=a21*y+a22*ydot+b21*dble(acc(i))+b22*dble(acc(i+1))
        y=y1    ! y is the oscillator output at time corresponding to index i
        z=dabs(y)
        if (z > rd) rd=z
        z1 = dabs(ydot)                   ! for rv
        if (z1 > rv) rv = z1            ! for rv
        ra = -d3*ydot -om2*y1            ! for aa
        z2 = dabs(ra)                     ! for aa
        if (z2 > aa) aa = z2            ! for aa
        
      end do
      
      return
      end
!----------------- END RDRVAA -----------------------------

! --------------------------- BEGIN GET_LUN ----------------
      subroutine get_lun(lun)

! Finds a logical unit number not in use; returns
! -1 if it cannot find one.

! Dates -- 05/19/98 - Written by D. Boore, following
!                     Larry Baker's suggestion
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      logical isopen
      do i = 99,10,-1
        inquire (unit=i, opened=isopen)
        if(.not.isopen) then
          lun = i
          return
        end if
      end do
      lun = -1

      return
      end
! --------------------------- END GET_LUN ----------------
     
! --------------------- BEGIN UPSTR ----------------------------------
      Subroutine UPSTR ( text )
! Converts character string in TEXT to uppercase
! Dates: 03/12/96 - Written by Larry Baker
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

!
      Implicit   None
!
      Character  text*(*)
!
      Integer    j
      Character  ch
!
      Do 1000 j = 1,LEN(text)
         ch = text(j:j)
         If ( LGE(ch,'a') .and. LLE(ch,'z') ) Then
            text(j:j) = CHAR ( ICHAR(ch) - ICHAR('a') + ICHAR('A') )
         End If
 1000    Continue
!
      Return
      End
! --------------------- END UPSTR ----------------------------------
! --------------------------- BEGIN TRIM_C -----------------------
      subroutine trim_c(cstr, nchar)

! strips leading and trailing blanks from cstr, returning the
! result in cstr, which is now nchar characters in length

! Strip off tabs also.

! Here is a sample use in constructing a column header, filled out with 
! periods:

!* Read idtag:
!        idtag = ' '
!        read(nu_in, '(1x,a)') idtag
!        call trim_c(idtag, nc_id)
!* Set up the column headings:
!        colhead = ' '
!        colhead = idtag(1:nc_id)//'......' ! nc_id + 6 > length of colhead

! Dates: 12/23/97 - written by D. Boore
!        12/08/00 - pad with trailing blanks.  Otherwise some comparisons
!                   of the trimmed character string with other strings
!                   can be in error because the original characters are left
!                   behind after shifting.  For example, here is a string
!                   before and after shifting, using the old version:
!                      col:12345
!                           MTWH  before
!                          MTWHH  after (but nc = 4).
!        03/21/01 - Check for a zero length input string
!        11/09/01 - Change check for zero length string to check for all blanks
!        10/19/09 - Strip off tabs
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)
!        02/14/18 - Use "len_trim" to check for cstr being all blank

      character cstr*(*)

      nend = len_trim(cstr)

      if(nend == 0) then
        nchar = 0
        return
      end if

! Replace tabs with blanks:

      do i = 1, nend
        if(ichar(cstr(i:i)) == 9) then
           cstr(i:i) = ' '
        end if
      end do



!      if(nend == 0) then
!        nchar = 0
!        return
!      end if

      do i = nend, 1, -1
        if (cstr(i:i) /= ' ') then
           nchar2 = i
           goto 10
        end if
      end do

10    continue

      do j = 1, nchar2
        if (cstr(j:j) /= ' ') then
          nchar1 = j
          goto 20
        end if
      end do

20    continue
   
      nchar = nchar2 - nchar1 + 1
      cstr(1:nchar) = cstr(nchar1: nchar2)
      if (nchar < nend) then
        do i = nchar+1, nend
          cstr(i:i) = ' '
        end do
      end if

      return
      end
! --------------------------- END TRIM_C -----------------------! ---------------------- BEGIN SKIP -------------------
      subroutine SKIP(lunit, nlines)
        if (nlines .lt. 1) then
          return
        else
          do i = 1, nlines
             read(lunit, *)
          end do
          return
        end if
      end
! ---------------------- END SKIP -------------------

! ---------------------- BEGIN Abs_MnMaxIdx ----------------------
      subroutine abs_mnmaxidx(a,nstrt,nstop,ninc,
     :                    amin,amax,indx_min,indx_max, abs_max) 

* A rewrite of mnmax, returning the indices of the min and max values
*
* Dates: 07/04/05 - Written by D. M. Boore, adapted from mnmaxidx
 
      real a(*)
      amax = a( nstrt)
      indx_max = nstrt
      amin=amax         
      indx_min = nstrt
      do i=nstrt,nstop,ninc
        if(a(i) .gt. amax) then
          amax=a(i)
          indx_max = i
        end if
        if(a(i) .lt. amin) then
          amin=a(i)
          indx_min = i
        end if
      end do
      if (abs(amin) .gt. abs(amax)) then
        abs_max = abs(amin)
      else
        abs_max = abs(amax)
      end if
      
      return                  
      end                     
! ---------------------- END Abs_MnMaxIdx ----------------------

! ---------------------- BEGIN MNMAX ----------------------
      subroutine mnmax(a,nstrt,nstop,ninc,amin,amax) 
!
! Dates: 09/07/84 - Written by D. Boore
!        11/02/12 - replace obsolete if statement 
! 
!
      dimension a(*)
      
      amax = a(nstrt)
      amin=amax 
      
      do i=nstrt,nstop,ninc

        if ( a(i) < amin) then
          amin=a(i)
        else if ( a(i) > amax) then
          amax=a(i) 
        end if
  
      end do
      
      return                  
      end                     
! ---------------------- END MNMAX ----------------------
* ------------------------------------------------------------- SMC_NSps
      subroutine smc_nsps(f_smc, npts, sps)

* Open smc file and obtain npts, sps of time series:

* Dates: 12/26/00 - Written by D. Boore, patterned after smc_npts.for
*        01/04/01 - Reset sps to -1.0 if unevenly spaced data

      integer int_head(48)
      real real_head(2)
      character f_smc*(*)

      call get_lun(nu_smc)
      open(unit=nu_smc,file=f_smc,status='unknown')
      call skip(nu_smc,11)
      read(nu_smc,'(8i10)') (int_head(i),i=1,48)
      npts = int_head(17)
      read(nu_smc,'(5e15.7)') (real_head(i),i=1,2)
      if (real_head(2) .eq. real_head(1)) then   ! unevenly sampled
        sps = -1.0
      else
        sps = real_head(2)
      end if

      close(nu_smc)        

      return
      end
* ------------------------------------------------------------- SMC_NSps

!------------------   Begin SMCRead   ----------------------------------
      subroutine SMCRead(f_in,  
     :                   tskip, tlength,
     :                   y, char_head, int_head, real_head, comments)

!  If tskip = 0.0, no data are skipped
!  If tlength = 0.0, only headers are read
!  If tlength < 0.0, read all data, less tskip
!  Otherwise tlength of data is read, after skipping tskip, unless 
!  tskip + tlength exceeds the length of the input time series, in which
!  case tlength is adjusted so that all data, less tskip, are read.  Thus
!  setting tlength to a large number (or setting it to a negative number)
!  guarantees that all data (minus tskip) are read.

! The program requires get_lun.for

! The output arrays char_head, int_head, real_head should be dimensioned
! as follows in the calling program:

!      character char_head(11)*80, comments(500)*80 ! "80" is the critical parameter
!      integer int_head(48)
!      real real_head(50)

!  Dates: 07/17/97 - Modified order of input parameters, to put input
!                    first (unit, file, tskip), followed by output.
!         12/18/97 - Read tlength
!         05/19/98 - Determine unit from within program; remove "unit"
!                    from argument list
!         03/09/99 - Change logic to determine amount of record to
!                    read and allow reading of all record, minus tskip,
!                    by setting tlength < 0.0.
!         05/18/99 - Deleted npts, sps from input parameter list.  Added 
!                    character array with comments string (allow for 
!                    50 comments).
!         12/16/99 - Renamed from ReadSMC
!         05/10/00 - Replaced
!                        tlength = float(npts_out/sps)
!                    with
!                        tlength = float(npts_out)/sps
!                    (the error and correction was pointed out by Wang Guoquan
!                    in email on 05/06/2000)
!         01/04/01 - Check for unevenly spaced data (sps = rnull) and
!                    return 0.0 in y
!         02/07/01 - If tlength = 0, only read headers
!         12/14/01 - Fixed logic for case when tlength < 0 and tskip > 0.
!                    Also, program used to reset tlength, but it does not do
!                    that now.  The user can compute tlength as 
!                    float(int_head(17))/real_head(2)
!         01/01/05 - Use "int" in calculation of npts (do not use
!                    implicit conversion from real to integer)
!         10/15/07 - Use higher precision input format if int_head(47) = 8
!         04/04/10 - Add call to trim_c, and use nc_f_in.
!         10/20/15 - Changed format of high precision input.
!                  - This was not necessary, as the old format statement can read
!                    either e14.7 or e14.7e1 (the new format being used in smcwrite for
!                    high precision).
!         12/30/15 - Replaced "int" with "nint" when computing nskip and npts2read
!                  - A few modernizations of syntax.
!         05/04/17 - replaced "float" with the generic function "real"

      real real_head(*)
      integer int_head(*)
      character*80 char_head(*), comments(*)
      character f_in*(*) 
      real y(*)

      
      
      call get_lun(nu)
      call trim_c(f_in, nc_f_in)
      open(unit= nu, file=f_in(1:nc_f_in), status='unknown')

      do i = 1, 11
        read(nu, '(a)') char_head(i)
      end do

      read(nu, '(8I10)') (int_head(i), i=1, 48)

      read(nu, '(5e15.7)') (real_head(i), i = 1, 50)

      sps = real_head(2)

      do i = 1, int_head(16)
        read(nu,'(a)') comments(i)
      end do

      if(real_head(2) == real_head(1)) then   ! unevenly sampled data

         int_head(17) = 2
         do i = 1, int_head(17)
           y(i) = 0.0
         end do

      else

! Skip into the trace:

        nskip = nint(tskip * sps)
!        nskip = int(tskip * sps)

! How much record to read?

        npts_in = int_head(17)
        npts2read = nint(tlength * sps)
!        npts2read = int(tlength * sps)

        if (tlength < 0.0) then  
          npts_out = npts_in - nskip
        else if (nskip + npts2read <= npts_in) then
          npts_out = npts2read
        else
          npts_out = npts_in - nskip
!          tlength = real(npts_out)/sps
        end if

        int_head(17) = npts_out
      
        if (tlength == 0.0) then
          return    ! only read headers
        else
          if (int_head(47) == 8) then
            read(nu, '(5(e14.7))') (y(i), i = 1, npts_out + nskip)
!            read(nu, '(5(e14.7e1))') (y(i), i = 1, npts_out + nskip)
          else
            read(nu, '(8(1pe10.4e1))') (y(i), i = 1, npts_out + nskip)
          end if
          do i = 1, npts_out
            y(i) = y( i + nskip)
          end do
        end if

      end if

      close(unit=nu)

      return
      end
!------------------   End SMCRead   ----------------------------------




! ------------------------------------------------------------------ skipcmnt
      subroutine skipcmnt(nu, comment, ncomments)

! Skip text comments in the file attached to unit nu, but save skipped 
! comments in character array comment.  Skip at least one line, and more as 
! long as the lines are preceded by "|" or "!".

! Dates: 04/16/01 - Written by D. Boore
!        12/07/01 - Added check for eof
!        11/04/03 - Use trim_c to trim possible leading blank
!        02/03/07 - Initialize comments to blank
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      character comment(*)*(*), buf*80

      ncomments = 0
100   buf = ' '
      read (nu,'(a)',end=999) buf
      call trim_c(buf,nc_buf)
      if (buf(1:1) .eq.'!' .or. buf(1:1) .eq.'|' .or. 
     :                     ncomments + 1 .eq. 1) then
        ncomments = ncomments + 1
        comment(ncomments) = ' '
        comment(ncomments) = buf(1:nc_buf)
        goto 100
      else 
        backspace nu
      end if

999   continue
 
      return
      end
! ------------------------------------------------------------------ skipcmnt
* ------------------------------------------------ Get_Path_From_System_Call
      subroutine get_path_from_system_call(path,nc_path)

* Get path using a system call

* Dates: 07/16/07 - Written by D. Boore

      character path*(*), buf_work*120

      call system("dir smctsplt.ctl > dir.lst")
      call get_lun(nu_path)
      open(nu_path,file='dir.lst',status='unknown')
      call skip(nu_path,3)
      buf_work = ' '
      read(nu_path,'(a)') buf_work
      call trim_c(buf_work, nc_buf_work)
      close(nu_path)
      
      path = ' '
      do i = 1, nc_buf_work
        if (buf_work(i:i) .eq. '\') then
          path(i:i) = '/'
        else
          path(i:i) = buf_work(i:i)
        end if
      end do
      call trim_c(path,nc_path)
  
      call system("del dir.lst")

      return
      end
* ------------------------------------------------ Get_Path_From_System_Call

! ---------------------------------------------------------------- smcpadf
      subroutine smcpadf(y, npts_in, sps, nlz, ntz, 
     :  replace_discarded_data_with_zeros, 
     :  taper_beginning, taper_end, 
     :  npts_out, izcl, izct, 
     :  remove_mean_before, remove_mean_after, 
     :  zcross_l, use_first, zcross_t, use_last,
     :  comments, ncomments)    

* Here is the padding action taken for some combinations of input parameters:
*
* zcross_l  use_first  pad action (similar for zcross_t, use_last)
*     T         T         pad with 0.0
*     T         F         pad with 0.0
*     F         T         pad with work(1)
*     F         F         pad with 0.0
 
* Dates: 06/04/04 - Replaced tlz, ttz with nlz, ntz.  Also, allow the 
*                   discarded data to be replaced with zeros (thus 
*                   preserving the length of the input time series).
*        03/13/06 - Remove f_smc from smcpadf
*        11/03/06 - If zcross_l or zcross_t = F and use_first or 
*                   use_last = T, pad with values equal to the last points.
*                   Allow this option to deal with cases such as filtering
*                   a displacement time series with a low-frequency drift, 
*                   which will have a step offset if pad with zeros at the
*                   end.  Be careful in in using this option for the first
*                   points, however (in a future version I may eliminate
*                   the option for the beginning, as generally the leading
*                   pad should be zero).  The need for this option arose
*                   when I tried to filter a displacement obtained by 
*                   integrating a zeroth-order corrected acceleration.
*                   The results were different than filtering the 
*                   acceleration before integration.
*        01/24/07 - Store the snipped portion of y_in in a work array.  This
*                   is to be sure that I am not overwriting the input data
*                   when shifting the time series.  Probably the existing
*                   code was OK, but on reading it some years after writing
*                   it, I find it difficult to follow.  It is much better to
*                   make things a bit less efficient, but easier to follow.
*        01/24/07 - Added option to taper end of time series before adding 
*                   trailing zero pad.
*        07/15/07 - Added option to taper beginning of time series before 
*                   adding leading zero pad.
*        07/17/07 - Corrected a subtle error:  the names for the input 
*                   parameters taper_beginning and taper_end were being used
*                   for the variable containing the taper, and thus their
*                   values were being reset to the last taper value.   I now
*                   use "taper" as the variable containing the taper value.
*        03/24/08 - Write leading zpads as a time as well as n points.
*        01/18/10 - Add comments above about padding
!        04/04/10 - Correct error that pad_value_l and pad_value_t could be
!                   undefined in some cases.

  
      logical remove_mean_before, remove_mean_after,
     :  zcross_l, use_first, zcross_t, use_last, 
     :  replace_discarded_data_with_zeros

      real y(*)
      character comments(*)*80

      real work(:)
      allocatable :: work

      parameter (small=1.0e-10)
!      parameter (small=1.0e-05)

      pi = 4.0*atan(1.0)

!* DEBUG
!      write(*,'(a)') ' Inside smcpadf:'
!
!      write(*,'(a)') ' nlz, ntz = '
!      write(*, *)      nlz, ntz
!
!      write(*,'(a)') ' zcross_l, zcross_t = '
!      write(*, *)      zcross_l, zcross_t
!
!      write(*,'(a)') ' use_first, use_last = '
!      write(*, *)      use_first, use_last
!
!      write(*,'(a,1x,l1)') 
!     :      ' remove_mean_before = ', remove_mean_before
!
!      write(*,'(a,1x,l1)') 
!     :      ' remove_mean_after = ', remove_mean_after
!
      if (remove_mean_before) then        
        n_start = 1
        n_stop = npts_in
        icorr_cumavg_before = 0
        call Get_Avg(y, n_start, n_stop, npts_in, 
     :                   sps, icorr_cumavg_before, avg_before)
        do i = n_start, n_stop
          y(i) = y(i) - avg_before
        end do
        n_start_before = n_start
        n_stop_before = n_stop
      end if

* Get peak motion:
      
      call mnmaxidx(y, 1, npts_in, 1,
     :             ymin, ymax, indx_min, indx_max) 
      peak_mtn = ymax
      if (abs(ymin) .gt. abs(ymax)) peak_mtn = ymin

* Define fraction small of peak:
      smidge = small * abs(peak_mtn)


      if (zcross_l) then ! Find first zero crossing
        izcl = -1
        if (abs(y(1)) .lt. smidge) then
          izcl = 1
        else
          do i = 2, npts_in
            if (y(i)*y(1) .lt. 0.0) then   ! a zero crossing has been found
* Now determine if the point just before the zero crossing is smaller in 
* amplitude than the point after.  If so, let it be the first point unless 
* the first value is smaller than this value, in which case y(1) is the 
* first value.
              if (abs(y(i)) .lt. abs(y(i-1))) then
                izcl = i
              else
                izcl = i - 1
                if (use_first) then
                  if (abs(y(i-1)) .gt. abs(y(1))) then
                    izcl = 1
                  end if
                end if
              end if
              go to 2345
            end if
          end do
2345      continue
        end if
      else
        izcl = 1
      end if
      if (zcross_t) then ! Find last zero crossing
        izct = -1
        if (abs(y(npts_in)) .lt. smidge) then
          izct = npts_in
        else
          do i = npts_in -1, 1, -1
            if (y(i)*y(npts_in) .lt. 0.0) then ! a zero crossing has been found
* Now determine if the point just after the zero crossing is smaller in 
* amplitude than the point before.  If so, let it be the last point unless 
* the last value is smaller than this value, in which case y(npts_in) is the 
* last value.
              if (abs(y(i)) .lt. abs(y(i+1))) then
                izct = i
              else
                izct = i+1
                if (use_last) then
                  if (abs(y(i+1)) .gt. abs(y(npts_in))) then
                    izct = npts_in
                  end if
                end if
              end if
              go to 5432
            end if
          end do
5432      continue
        end if
      else
        izct = npts_in
      end if

!      write(*,'(a)') ' izcl, izct = '
!      write(*,*)       izcl, izct

* izcl, izct = index in input smc file of zero crossings

* April's code is more compact, but I think mine is easier to read        
            
      if (replace_discarded_data_with_zeros) then
        nlznew = nlz + izcl - 1
        ntznew = ntz + npts_in - izct
      else
        nlznew = nlz
        ntznew = ntz
      end if

* Allocate a work array and save the snipped portion in this array

      npts_snip = izct - izcl + 1  ! number of original points between zeros
      allocate (work(npts_snip)) 
      do i = 1, npts_snip
        work(i) = y(i+izcl-1)
      end do
      
* Compute new npts_out:

      npts_out = npts_snip + nlznew + ntznew

* Indices of original data

        n_start = nlznew + 1
        n_stop = npts_snip + nlznew

* Compute and remove mean if specified:

      if (remove_mean_after) then        
        n_start4avg = 1
        n_stop4avg = npts_snip 
        icorr_cumavg_after = 1

        call Get_Avg(work, n_start4avg, n_stop4avg, npts_snip, 
     :                   sps, icorr_cumavg_after, avg_after)
        do i = 1, npts_snip
          work(i) = work(i) - avg_after
        end do
        n_start_after = n_start
        n_stop_after = n_stop
      end if

* Taper beginning, if desired:

      ntaper_beginning = taper_beginning*sps
      
      if (ntaper_beginning .gt. 0) then
      
!* DEBUG
!        print *,' ntaper_beginning = ', ntaper_beginning
!* DEBUG
      
        do i = 1, ntaper_beginning + 1
        
           ishift = i - (1 - ntaper_beginning)
           arg = pi*float(ishift)/float(ntaper_beginning)
           taper = 0.5*(1.0+cos(arg))
           
           work(i) = taper * work(i)
           
        end do
        
      end if



* Taper end, if desired:

      ntaper_end = taper_end*sps
      
      if (ntaper_end .gt. 0) then
      
!* DEBUG
!        print *,' ntaper_end = ', ntaper_end  
!* DEBUG
      
        do i = npts_snip - ntaper_end, npts_snip
        
           ishift = i - (npts_snip - ntaper_end)
           arg = pi*float(ishift)/float(ntaper_end)
           taper = 0.5*(1.0+cos(arg))
           
           work(i) = taper * work(i)
           
        end do
        
      end if



* Store work in shifted location in y

      do i = 1, npts_snip
        y(i + nlznew) = work(i)
      end do
      
* And pad with leading zeros:

      pad_value_l = 0.0
      if ( nlznew .gt. 0) then
        if ((.not. zcross_l) .and. use_first) then
          pad_value_l = work(1)
        else
          pad_value_l = 0.0
        end if
        
        do i = 1, nlznew
          y(i) = pad_value_l
        end do
      end if

* And pad with trailing zeros:

      pad_value_t = 0.0
      if ( ntznew .gt. 0) then
        if ((.not. zcross_t) .and. use_last) then
          pad_value_t = work(npts_snip)
        else
          pad_value_t = 0.0
        end if

        do i = 1, ntznew
          y(npts_snip + nlznew + i) = pad_value_t
        end do
      end if

      deallocate(work)

* Store  comments:
      ncomments = ncomments + 1
      write(comments(ncomments),'(a)') 
     :     '| Added pads:'
      if (remove_mean_before) then        
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i1,a,1x,1p,e11.4)') 
     :   '| Using icorr_cumavg = ', icorr_cumavg_before, 
     :   ', determined and removed the mean', avg_before
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i5,1x,a,1x,i5,a)') 
     :   '| between indices ', n_start_before, ' and ',
     :        n_stop_before, ' before padding with pad_value'
      else
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i1,a,1x,1p,e11.4)') 
     :   '| No mean removed before adding pad' 
      end if
      if (remove_mean_after) then        
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i1,a,1x,1p,e11.4)') 
     :   '| Using icorr_cumavg = ', icorr_cumavg_after, 
     :   ', determined and removed the mean', avg_after
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i5,1x,a,1x,i5,a)') 
     :   '| between indices ', n_start_after, ' and ',
     :        n_stop_after, ' after padding with pad_value'
      else
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i1,a,1x,1p,e11.4)') 
     :   '| No mean removed after adding pads' 
      end if
      
      if (ntaper_beginning .gt. 0) then
!* DEBUG
!        print *,' In write comments:'
!        print *,' ntaper_beginning = ', ntaper_beginning  
!* DEBUG
      
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i7,a)')
     :     '| Taper beginning ', ntaper_beginning,
     :     ' points before adding leading pad'
      end if
          
      if (ntaper_end .gt. 0) then
!* DEBUG
!        print *,' In write comments:'
!        print *,' ntaper_end = ', ntaper_end 
!* DEBUG
      
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i7,a)')
     :     '| Taper last ', ntaper_end,
     :     ' points before adding trailing pad'
      end if
          
      ncomments = ncomments + 1
      if (replace_discarded_data_with_zeros) then
        write(comments(ncomments),'(a,1x,i7,a)') 
     :     '| The program replaced ', izcl - 1, 
     :      ' points from start of record with pad value'
      else
        write(comments(ncomments),'(a,1x,i7,a)') 
     :     '| The program discarded ', izcl - 1, 
     :      ' points from the start of the record'
      end if
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,i7,a,
     :                            f7.3,a)') 
     :     '|   and added ', nlz, ' leading values ( ', 
     :                     float(nlz)/sps, ' s), using'
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,l1,a,1x,l1)') 
     :     '|   zcross_l = ', zcross_l,
     :        ' and use_first = ', use_first
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,1p, e10.3)') 
     :     '|   and pad_value = ', pad_value_l
      ncomments = ncomments + 1
      if (replace_discarded_data_with_zeros) then
        write(comments(ncomments),'(a,1x,i7,a)') 
     :     '| The program replaced ', npts_in - izct, 
     :      ' points from end of record with pad_value'
      else
        write(comments(ncomments),'(a,1x,i7,a)') 
     :     '| The program discarded ', npts_in - izct, 
     :      ' points from the end of the record'
      end if
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,i7,a)') 
     :     '|   and added ', ntz, ' trailing values, using'
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,l1,a,1x,l1)') 
     :     '|   zcross_t = ', zcross_t,
     :        ' and use_last = ', use_last
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,1p, e10.3)') 
     :     '|   and pad_value = ', pad_value_t
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,i7)') 
     :     '| The total number of points in the input time'//
     :        ' series = ', npts_in
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,i7)') 
     :     '| The total number of points in the '//
     :       'pad-stripped series = ', npts_out - nlz - ntz

      return
      end
! ---------------------------------------------------------------- smcpadf


! ------------------------------------------------------  begin construct_filename_extension
      subroutine construct_filename_extension
     :      (tskip, tlength, option, 
     :       flc, nslope_lc, icaus_lc,
     :       fhc, nslope_hc, icaus_hc, icaus_bn,
     :       filter_type, nc_filter_type,
     :       taper_beginning, taper_end, 
     :       fn_ext_c, nc_fn_ext_c)
     
! Construct a filename extension based on the blpadflt parameter
! "option" (10=no baseline) and the blpadflt filtering  
! parameters.  At this time I do not include baseline correction or
! tapering details, as these are less commonly used than the filtering

! Dates: 12/08/09 - Add taper info to filename
!        01/25/10 - Correct error in construct_filename_extension.for when auto naming the 
!                   output file if only remove mean (no filtering).
!        03/11/10 - Correct error of not calling trim_c for fn_ext_c
!        03/20/10 - Allow for a notch filter.
!        12/04/10 - Add tskip, tlength to name
!        05/16/14 - Modernize boolean statements
!                 - Introduce option==11 to mean no baseline, and no mean removed.
!                   I needed this in order to make the filename of the unfiltered velocity
!                   time series in the filt_plot_lc output indicate correctly whether
!                   or not a mean had been removed.
!        07/11/17 - Add more filter possibilities
!        07/13/17 - Determine filter type in calling program
     
      character fn_ext_c*(*), d_c*6, freql_c*6, freqh_c*6, ns_c*2
      integer option
      character filter_type*(*)
      
      fn_ext_c = ' '
      fn_ext_c = '.tskp'

      call trim_c(fn_ext_c, nc_fn_ext_c)
      if (tskip < 10.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+3),
     :             '(f3.1)') tskip
      else if (tskip < 100.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+4),
     :             '(f4.1)') tskip
      else if (tskip < 1000.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+5),
     :             '(f5.1)') tskip
      else if (tskip < 10000.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+6),
     :             '(f6.1)') tskip
      else 
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+7),
     :             '(f7.1)') tskip
      end if
      call trim_c(fn_ext_c, nc_fn_ext_c)
      
      fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//'.tread'
      call trim_c(fn_ext_c, nc_fn_ext_c)
      if (tlength <  0.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+4),
     :             '(f4.1)') tlength
      else if (tlength < 10.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+3),
     :             '(f3.1)') tlength
      else if (tlength < 100.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+4),
     :             '(f4.1)') tlength
      else if (tlength < 1000.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+5),
     :             '(f5.1)') tlength
      else if (tlength < 10000.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+6),
     :             '(f6.1)') tlength
      else 
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+7),
     :             '(f7.1)') tlength
      end if
      call trim_c(fn_ext_c, nc_fn_ext_c)
     
      if (icaus_lc == 1 .and. icaus_hc == 1) then
        icaus = 1
      else
        icaus = 0
        end if


      
      if (filter_type(1:nc_filter_type) == 'NF') then  ! no filter
        if (option == 10 .or. option == 11) then ! no baseline correction
          if (option == 11) then
            fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//'.xrm'
          else
            fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//'.rm'
          end if
        else  ! baseline correction
          fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//'.blc'
        end if
        call trim_c(fn_ext_c, nc_fn_ext_c)

      else ! filtering
      
        if (filter_type(1:nc_filter_type) /= 'NF') then  ! A filter has been applied

          if (filter_type(1:nc_filter_type) == 'LC') then ! locut only
            ns_c = ' '
            write(ns_c(1:2),'(i2.2)') nslope_lc
            call freq2char(flc, freql_c)
            if (icaus_lc == 0) then
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.alc'//freql_c(1:6)//'ns'//ns_c(1:2)
            else
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.clc'//freql_c(1:6)//'ns'//ns_c(1:2)
            end if
          else if (filter_type(1:nc_filter_type) == 'HC') then ! hicut only
            ns_c = ' '
            write(ns_c(1:2),'(i2.2)') nslope_hc
            call freq2char(fhc, freqh_c)
            if (icaus_hc == 0) then
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.ahc'//freqh_c(1:6)//'ns'//ns_c(1:2)
            else
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.chc'//freqh_c(1:6)//'ns'//ns_c(1:2)
            end if
          else if (filter_type(1:nc_filter_type) == 'NOTCH') then ! notch
            d = real(nslope_lc)*10.0**(-real(nslope_hc))
            call freq2char(flc, freql_c)
            call freq2char(d, d_c)
            if (icaus_bn == 1) then  !causal
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :       '.cnotch'//freql_c(1:6)//'d'//d_c(1:6)
            else
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :       '.anotch'//freql_c(1:6)//'d'//d_c(1:6)
            end if
          else if (filter_type(1:nc_filter_type) == 'BAND') then ! bandpass          
            nslope_bp = max0(nslope_lc, nslope_hc)
            ns_c = ' '
            write(ns_c(1:2),'(i2.2)') nslope_bp
            call freq2char(flc, freql_c)
            call freq2char(fhc, freqh_c)
            if (icaus_bn == 0) then
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :       '.abp'//freql_c(1:6)//'-'//freqh_c(1:6)//'ns'//ns_c(1:2)
            else
                fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :       '.cbp'//freql_c(1:6)//'-'//freqh_c(1:6)//'ns'//ns_c(1:2)
            end if
          else if (filter_type(1:nc_filter_type) == 'LCHC') then ! locut followed by hicut
            ns_c = ' '
            write(ns_c(1:2),'(i2.2)') nslope_lc
            call freq2char(flc, freql_c)
            if (icaus_lc == 0) then
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.alc'//freql_c(1:6)//'ns'//ns_c(1:2)
            else
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.clc'//freql_c(1:6)//'ns'//ns_c(1:2)
            end if
            call trim_c(fn_ext_c, nc_fn_ext_c)
            ns_c = ' '
            write(ns_c(1:2),'(i2.2)') nslope_hc
            call freq2char(fhc, freqh_c)
            if (icaus_hc == 0) then
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.ahc'//freqh_c(1:6)//'ns'//ns_c(1:2)
            else
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.chc'//freqh_c(1:6)//'ns'//ns_c(1:2)
            end if
          else
            write(*,'(a)') 
     :          ' In construct_filename_extension, '//
     :          'unknown filter type; quitting!!! '
            stop
          end if
        end if

        call trim_c(fn_ext_c, nc_fn_ext_c)

! Add taper info if an acausal filter:
 
        if (icaus_bn == 0 .or. icaus_lc == 0 .or. icaus_hc == 0) then 
      
          fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//'.taper_'
          call trim_c(fn_ext_c, nc_fn_ext_c)
        
          if (taper_beginning >= 10.0) then
            write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+4),
     :             '(f4.1)') taper_beginning
          else
            fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+1) = '_'
            write(fn_ext_c(nc_fn_ext_c+2:nc_fn_ext_c+4),
     :             '(f3.1)') taper_beginning
          end if
        
          fn_ext_c(nc_fn_ext_c+5:nc_fn_ext_c+5) = '_'
          call trim_c(fn_ext_c, nc_fn_ext_c)
          if (taper_end >= 10.0) then
            write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+4),
     :             '(f4.1)') taper_end
          else
            fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+1) = '_'
            write(fn_ext_c(nc_fn_ext_c+2:nc_fn_ext_c+4),
     :             '(f3.1)') taper_end
          end if
 
          call trim_c(fn_ext_c, nc_fn_ext_c)
        end if

      end if 
      
      call trim_c(fn_ext_c, nc_fn_ext_c)
      
      return
      end
! ------------------------------------------------------  end construct_filename_extension

      
! ------------------------------------------------------  begin freq2char
      subroutine freq2char(f, freq_c)
        character freq_c*6
        real f
        freq_c = '000000'
        if (f.lt.10.0) then
          write(freq_c(2:6),'(f5.3)') f
        else  
          write(freq_c(1:6),'(f6.3)') f
        end if
        
      return
      end
! ------------------------------------------------------  end freq2char      
!----------------- BEGIN Acc2VD -----------------------------
      subroutine acc2vd(acc, npts, dt, rmv_trnd, vel0, dis0, vel, dis)


* Compute velocity and displacement time series from acceleration,
* assuming that the acceleration
* is represented by straight lines connecting the digitized values.

* Dates: 02/09/99 - Written by D.M. Boore
*        01/07/00 - Added initial velocity and displacements (v0, d0).
*                   This also requires the addition of a linear trend in
*                   displacement.
*                   Also, Bill Joyner, Chris Stephens, and I considered how to
*                   handle the first point.  Assuming v0 = 0, BAP uses a 
*                   trapezoidal rule such that the first vel(1) = 0.5*dt*a(1),
*                   but in acc2vd, vel(1) = 0.  In effect, BAP starts the
*                   integration at -dt rather than 0.0, with the result that
*                   because vel(1) depends on dt, vel(1) will change if the
*                   digitization interval is changed.  We agreed that this is 
*                   not correct.
*        12/18/02 - Some minor changes, such as specifying 0.0d0 rather 
*                   than 0.0 and using the function "dble" in various places
*        10/15/07 - Rename "d0" to "dis0" so as not to confuse with "d0"
*                   in "0.0d0".  For consistency, change "v0" to "vel0".
*                   More importantly, restructure the calculations so as not to
*                   lose the double-precision estimates of velocity when
*                   accumulating the displacement, and abandon the calculation 
*                   of disp based on integrating straightlines in acceleration, 
*                   using a simple trapezoidal rule instead.  These changes were
*                   made in an attempt to track down a drift in displacement
*                   for the HHE component of the KYTH recording of the
*                   Kythera earthquake recorded on a station of the EGELADOS
*                   velocity-sensor network.   The drift did not appear when I
*                   integrated the acceleration in Excel, and the SD computed 
*                   using SMC2RS also showed no indication of the drift (the
*                   long-period SD equals the peak displacement during the
*                   strong part of the shaking).   Unfortunately, the changes
*                   made today did not eliminate the drift.   My next check is
*                   to see if the 5 digit resolution in the smc files might
*                   be responsible (the data given to me by Andreas Skarlatoudis
*                   were in column files with 7 digits of resolution, with a maximum
*                   exponent of 9).  When I 
*                   opened the asc file made by smc2asc in Excel and computed dis, 
*                   I found the same trend as in the smc2vd file. This tells me
*                   that the problem is in the asc2smc conversion (but why the
*                   SD computed using the smc files shows no sign of the drift
*                   is a mystery to me).  I eventually confirmed that the problem
*                   is the limited resolution of the standard format; I have added
*                   an option to the smc utility programs to use a higher-resolution
*                   format (converting all of the utility programs will take some
*                   time---I am only doing those that I currently need right now).
!        02/06/11 - Added comments to make it clear that now use trapezoidal integration.
!                 - Delete "sngl(cumv)" and "sngl(cumd)" for vel(1) and dis(1)
!        02/07/11 - Call a subroutine to do the integration
!        06/01/11 - Delete the include statement for the integration routine; it should be placed in the programs that
!                   call acc2vd instead.  The reason for this is because
!                   the subroutine acc2vd is also included in \smsim\td_subs, and a compile
!                   error can occur if the \forprogs folder with the integration routine
!                   is not available.  This means that the trapezoidal_integration.for must be available in smsim.
!                   I guarantee this by including it in make_td_subs_file.bat, which should be called
!                   in time domain cl*.bat files.

      real acc(*), vel(*), dis(*) 
      logical rmv_trnd
      double precision cumv, cumd, a1, a2, v1, v2,
     : ddt, ddt_2, ddtdt_6

      if (rmv_trnd) then      
* remove trend first (straight line between first and last points)
* Note: acc is replaced with detrended time series
*        call dcdt(acc, dt, npts, 1, npts, .false., .true.)  ! old routine,
*                                                         ! gives steps at ends

         call rmvtrend(acc, npts)
      end if

* compute velocity and displacement

      call Trapezoidal_Integration(acc, npts, dt, vel0, vel)
      call Trapezoidal_Integration(vel, npts, dt, dis0, dis)
      
!      ddt     = dble(dt)
!      ddt_2   = 0.5d0 * dble(dt)
!      
!!      ddtdt_6 = dble(dt)*dble(dt)/6.0d0
!
!      cumv = 0.0d0
!      cumd = 0.0d0
!
!      v1 = 0.0d0
!      v2 = 0.0d0
!      
!!      vel(1) = sngl(cumv) + vel0
!      dis(1) = sngl(cumd) + dis0
!
!      vel(1) = vel0
!      dis(1) = dis0
!
!      do j=2,npts
!      
!        a1 = dble(acc(j-1))
!        a2 = dble(acc(j))
!        
!        cumv = cumv + (a1 + a2)*ddt_2
!        v1 = v2
!        v2 = cumv + dble(vel0)  ! the vel0 term is the constant of integration
!        vel(j) = sngl(v2)
!        
!        cumd = cumd + (v1 + v2)*ddt_2
!        dis(j) = sngl(cumd) + dis0  ! the dis0 term is the constant of integration
!        
!! Legacy code, when I used analytical formulas based
!! on representing the acceleration as a series of straightline segments.
!!        cumd = cumd + v1*ddt + (2.0*a1 + a2)*ddtdt_6
!        dis(j) = sngl(cumd) + dis0  ! linear term accounted for in v1*ddt
!        dis(j) = sngl(cumd) + v0*float(j-1)*dt + dis0
!
!      end do

      return
      end
!----------------- END Acc2VD -----------------------------



! ----------------------------- BEGIN RMVTREND ----------------
      subroutine rmvtrend(y, n)

! Removes a straightline fit to first and last points, replacing
! the input array with the detrended array

! Dates: 02/09/99 - written by D. Boore
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)


      real y(*)

      y1 = y(1)
      y2 = y(n)
      slope = (y2 - y1)/float(n-1)

      do i = 1, n
        y(i) = y(i) - (y1 + slope*float(i-1))
      end do

      return
      end
! ----------------------------- END RMVTREND ----------------


! ------------------------------------------------------------- SMC_Npts
      subroutine smc_npts(f_smc, npts, sps)

* Open smc file and obtain length of time series:

* Dates: 12/07/00 - Written by D. Boore
*        07/17/03 - Read all of int_head and real_head, and add sps to output
!        04/04/10 - Add trim_c, and use nc_f_smc.

      integer int_head(48)
      real real_head(50)
      character f_smc*(*)

      call get_lun(nu_smc)
      call trim_c(f_smc,nc_f_smc)
      open(unit=nu_smc,file=f_smc(1:nc_f_smc),status='unknown')
      call skip(nu_smc,11)

      read(nu_smc, '(8I10)') (int_head(i), i=1, 48)
      npts = int_head(17)

      read(nu_smc, '(5e15.7)') (real_head(i), i = 1, 50)
      sps = real_head(2)

      close(nu_smc)        

      return
      end
! ------------------------------------------------------------- SMC_Npts  

! ---------------------- BEGIN MNMAXIDX ----------------------
      subroutine mnmaxidx(a,nstrt,nstop,ninc,
     :                    amin,amax,indx_min,indx_max) 

! A rewrite of mnmax, returning the indices of the min and max values
!
! Dates: 10/05/99 - written by D. M. Boore
!        05/16/02 - Replaced "dimension a(1)" with "real a(*)"
!        11/03/12 - Replace if-then with if--then-else

      real a(*)
      amax = a( nstrt)
      indx_max = nstrt
      amin=amax         
      indx_min = nstrt
      do i=nstrt,nstop,ninc
      
        if (a(i) > amax) then
          amax=a(i)
          indx_max = i
        else if (a(i) < amin) then
          amin=a(i)
          indx_min = i
        end if
        
      end do
      
      return                  
      end                     
! ---------------------- END MNMAXIDX ----------------------

! ------------------------------------------------------------------ Get_Avg
      subroutine Get_Avg(a, 
     :  n_start, n_stop, npts, sps, icorr_cumavg, avg)

! Find mean of a between the two indices

! If itype = 1, then half weight is given to the first and last points
! (as it would be if computing velocity when the data before and after the
! indices are 0.0).

! Also consider (no confidence limits, however):

!      SUBROUTINE avg_std(data,n,avg,std,itype)
      
! Computes mean and standard deviation for an array of data.
! This is based on the Numerical Recipes program avevar.
! Itype specifies the type of mean:
! itype = 1: arithmetic mean, std = standard deviation
! itype = 2: geometric mean, std = factor by which mean is divided
!                                  and multiplied to yield the
!                                  equivalent of minus and plus
!                                  one standard deviation

! Or

!      subroutine mean_std_cl(a,npts,mean,std,seom,cl70,cl90,cl95,cl99)
! Computes mean, standard deviation, and confidence limits of the array entries





! Dates: 12/17/02 - Written by D. Boore
!        12/18/02 - Add icorr_cumavg to try to get program smc_pad to give 0.0
!                   velocity at the end of the data segment
!        12/18/02 - Rethinking computation of average when want to
!                   apply it to velocity case (looking at acc2vd.for algorithm)
!        05/01/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      real a(*)
      real*8 cum_avg

      avg = 0.0
!      if (t4mean_start .lt. 0.0) t4mean_start = 0.0
!      if (t4mean_stop .gt. tend) t4mean_stop = tend
!      n_start = nint(sps * t4mean_start + 1.1)
!      n_stop = nint(sps * t4mean_stop + 1.1)
      if (n_start .lt. 0)    n_start = 1
      if (n_stop .gt. npts) n_stop = npts
      n_4mean = n_stop - n_start + 1
!      t4mean = float(n_4mean -1)/sps
      cum_avg = 0.0
      do i = n_start, n_stop
        cum_avg = cum_avg + dble(a(i))
      end do

      if (icorr_cumavg .eq. 1) then  ! Effectively ignore icorr_cumavg now
!        cum_avg = cum_avg - 0.5d00*(dble(a(n_start))+dble(a(n_stop)))         
!        avg = sngl(cum_avg/dble(float(n_4mean-1)))
        avg = sngl(cum_avg/dble(float(n_4mean)))
      else
        avg = sngl(cum_avg/dble(float(n_4mean)))
      end if

      return
      end
! ------------------------------------------------------------------ Get_Avg
!--------------------- BEGIN GET_DATE  ---------------------
      subroutine get_date(date_c)

! Dates: 04/29/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)


      implicit none


      character date_c*(*)
!      integer*2 iyr, imon, iday
      character datx*8, timx*10


! Microsoft compiler version:
!      call GETDAT (iyr, imon, iday)


!      date_c = ' '         ! character date_c*10


!      date_c(3:3) = '/'
!      date_c(6:6) = '/'


!      write(date_c(1:2),'(i2.2)') imon      
!      write(date_c(4:5),'(i2.2)') iday      
!      write(date_c(7:10),'(i4.4)') iyr      


! Lahey compiler version:
!      call DATE(date_c)   ! character date_c*8, but *10 is OK


! Standard Fortran 90 intrinsic Subroutine DATE_AND_TIME
      call DATE_AND_TIME( datx, timx )
! Date is returned as 'CCYYMMDD'
! Time is returned as 'hhmmss.sss'
! date_c should be declared as
! character date_c*10
      date_c = ' '
      date_c = datx(5:6) // '/' // datx(7:8) // '/' // datx(1:4)


      return
      end
!--------------------- END GET_DATE  ---------------------!--------------------- BEGIN GET_TIME  ---------------------
      subroutine get_time(time_c)

! Dates: 04/29/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      implicit none


      character time_c*(*)
!      integer*2 ihr, imin, isec, i100th
      character datx*8, timx*10


! Microsoft compiler version:
!      call GETTIM (ihr, imin, isec, i100th)


!      time_c = ' '             ! character time_c*11 for both compilers


!      time_c(3:3) = ':'
!      time_c(6:6) = ':'
!      time_c(9:9) = '.'


!      write(time_c(1:2),'(i2.2)') ihr      
!      write(time_c(4:5),'(i2.2)') imin      
!      write(time_c(7:8),'(i2.2)') isec      
!      write(time_c(10:11),'(i2.2)') i100th


! Lahey compiler version:
!      call TIME(time_c)


! Standard Fortran 90 intrinsic Subroutine DATE_AND_TIME
      call DATE_AND_TIME( datx, timx )
! Date is returned as 'CCYYMMDD'
! Time is returned as 'hhmmss.sss'
! time_c should be declared as
! character time_c*11
      time_c = ' '
      time_c = timx(1:2)//':'//timx(3:4) // ':' // timx(5:9)


      return
      end
!--------------------- END GET_TIME  --------------------- 
!----------------- BEGIN Trapezoidal_Integration -----------------------------
      subroutine Trapezoidal_Integration(y, npts, dt, yint_0, yint)


* Integrate a time series y with initial condition yint_0

 
* Dates: 02/07/11 - Written by D.M. Boore, patterned after acc2vd.for
      real y(*), yint(*) 
      double precision cumv, y1, y2, v1, v2,
     : ddt, ddt_2 

 
* Integrate

      ddt     = dble(dt)
      ddt_2   = 0.5d0 * dble(dt)
 
      cumv = 0.0d0
 
      v1 = 0.0d0
      v2 = 0.0d0
      
      yint(1) = yint_0
 
      do j=2,npts
      
        y1 = dble(y(j-1))
        y2 = dble(y(j))
        
        cumv = cumv + (y1 + y2)*ddt_2
        v1 = v2
        v2 = cumv + dble(yint_0)  ! the yint_0 term is the constant of integration
        yint(j) = sngl(v2)
        
      end do

      return
      end
!----------------- END Trapezoidal_Integration -----------------------------


! -------------------------------------------------------------------- FILTER
      subroutine filter(a, npts, dt, flc, nroll_lc, icaus_lc,
     :                  fhc, nroll_hc, icaus_hc, icaus_bn, d4notch, 
     :                  filter_type, nc_filter_type)

! Butterworth filter time series stored in 'a', according to the following:
! if fhc = 0 = flc       : no filtering
! if fhc = 0             : low-cut filter
! if flc = 0             : high-cut filter
! if neither flc, fhc = 0: band-pass filter
! if flc = fhc /= 0, a notch filter, where the width parameter d is given by real(nslope_lc)*10**(-real(nslope_hc))

! "bn" in "icaus_bn" means "Band" "Notch"; icaus_bn = 1 for a band or notch filter
! only if icaus_lc = icaus_hc = 1.

!
! Dates: 09/11/00 - Written by D. Boore
!        02/04/01 - Added separate nroll for lc and hc filters; for
!                   the bandpass filter the max of nroll_lc and nroll_hc 
!                   is used.  In the future, the bandpass subroutine may be 
!                   changed to allow for separate rolloffs at low and high
!                   frequencies.
!        02/20/10 - Added notch filter, and the d parameter for this filter
!        07/11/17 - Allow the low- and high-cut filters to be either causal or acausal, independently
!                   (previously they had to both be one or the other).
!        07/13/17 - Change input parameters (determine filter type in calling program)
      
      real a(*)
      character filter_type*(*)

      if (filter_type(1:nc_filter_type) == 'NF') then
        return
      else if (filter_type(1:nc_filter_type) == 'LC') then
        call locut(a, npts, flc, dt, nroll_lc, icaus_lc)
      else if (filter_type(1:nc_filter_type) == 'HC') then
        call hicut(a, npts, fhc, dt, nroll_hc, icaus_hc)
      else if (filter_type(1:nc_filter_type) == 'LCHC') then
        call locut(a, npts, flc, dt, nroll_lc, icaus_lc)
        call hicut(a, npts, fhc, dt, nroll_hc, icaus_hc)
      else if (filter_type(1:nc_filter_type) == 'BAND') then
        nroll_bp = max0(nroll_lc, nroll_hc)
        call band(a, npts, flc, fhc, dt, nroll_bp, icaus_bn)  
      else if (filter_type(1:nc_filter_type) == 'NOTCH') then
        call notch(a, npts, flc, dt, d4notch, icaus_bn)
      else
        write(*,'(a)') ' In filter, unknown filter type; quitting!!! '
        stop
      end if

      return
      end
! -------------------------------------------------------------------- FILTER    

!* ---------------------------------------------------------------------- BAND
      subroutine band(s,nd,f1,f2,delt,nroll,icaus)
c  Butterworth bandpass filter order 2*nroll (nroll<=8) (see Kanasewich, 
c    Time Sequence Analysis in Geophysics, Third Edition, 
c    University of Alberta Press, 1981)
c  written by W.B. Joyner 01/07/97
c  causal if icaus.eq.1 - zero phase shift otherwise
c  s(j) input = the time series to be filtered - output = the 
c    filtered series
c  dimension of s(j) must be as least as large as the larger of 
c    the following:
c    nd+3.0*float(nroll)/(f1*delt)
c    nd+6.0*float(nroll)/((f2-f1)*delt)
c  nd = the number of points in the time series
c  f1, f2 = the cutoff frequencies
c  delt = the timestep

* Dates: xx/xx/xx - Written by Bill Joyner
*        09/12/00 - Changed "n" to "nroll" to eliminate confusion with
*                   Kanesewich, who uses "n" as the order (=2*nroll), and
*                   cleaned up appearance of code by using spaces, indents, etc.
*        09/12/00 - double precision statements added so that the output
*                   series has sufficient precision for double integration.
*        11/08/00 - Increased dimension of s from 50000 to 100000
*        02/14/01 - Modified/corrected real numbers function calls to 
*                   double precision - cds
*        02/22/01 - Removed dimension of s (it is up to the user to specify
*                   it properly)
!        06/26/14 - Adopted some minor suggested changes by Chris Stephens

      real s(*)
      real*8 fact(16),b1(16),b2(16)
      real*8 pi,w1,w2,xp,yp,x1,x2,y1,y2
      real*8 pre, pim, argre, argim, rho, theta, sjre, sjim
      real*8 bj, cj, con

      if(f1.eq.0..or.f1.ge.f2) return  ! Changed "f1.eq.f2" to "f1.ge.f2" on 06/26/14

      pi=4.0d0*datan(1.0d0)
      w1=2.0d0*pi*f1
      w1=2.0d0*dtan(w1*delt/2.0d0)/delt
      w2=2.0d0*pi*f2
      w2=2.0d0*dtan(w2*delt/2.0d0)/delt

      do k=1,nroll
        pre=-dsin(pi*dfloat(2*k-1)/dfloat(4*nroll))
        pim=dcos(pi*dfloat(2*k-1)/dfloat(4*nroll))
        argre=(pre**2-pim**2)*(w2-w1)**2/4.0d0-w1*w2
        argim=2.0d0*pre*pim*(w2-w1)**2/4.0d0
        rho=(argre**2+argim**2)**(1.0d0/4.0d0)
        theta=pi+datan2(argim,argre)/2.0d0
        do i=1,2
          sjre=pre*(w2-w1)/2.0d0+(-1)**i*rho*(-dsin(theta-pi/2.0d0))
          sjim=pim*(w2-w1)/2.0d0+(-1)**i*rho*dcos(theta-pi/2.0d0)
          bj=-2.0d0*sjre
          cj=sjre**2+sjim**2
          con=1.0d0/(2.0d0/delt+bj+cj*delt/2.0d0)
          fact(2*k+i-2)=(w2-w1)*con
          b1(2*k+i-2)=(cj*delt-4.0d0/delt)*con
          b2(2*k+i-2)=(2.0d0/delt-bj+cj*delt/2.0d0)*con
        end do
      end do

      np2=nd

      if(icaus.ne.1) then
        npad=3.0*float(nroll)/(f1*delt)
        if( npad .lt. 6.0*float(nroll)/((f2-f1)*delt) ) then
          npad=6.0*float(nroll)/((f2-f1)*delt)
        end if
        np1=nd+1
        np2=nd+npad
        do j=np1,np2
          s(j)=0.0
        end do
      end if

      do k=1,2*nroll
        x1=0.0d0
        x2=0.0d0
        y1=0.0d0
        y2=0.0d0
        do j=1,np2
          xp=s(j)
          yp=fact(k)*(xp-x2)-b1(k)*y1-b2(k)*y2
          s(j)=yp
          y2=y1
          y1=yp
          x2=x1
          x1=xp
        end do
      end do

      if(icaus.ne.1) then
        do k=1,2*nroll
          x1=0.0d0
          x2=0.0d0
          y1=0.0d0
          y2=0.0d0
          do j=1,np2
            xp=s(np2-j+1)
            yp=fact(k)*(xp-x2)-b1(k)*y1-b2(k)*y2
            s(np2-j+1)=yp
            y2=y1
            y1=yp
            x2=x1
            x1=xp
          end do
        end do
      end if

      return

      end
!* ---------------------------------------------------------------------- BAND


!* --------------------------------------------------------------------- HICUT
      subroutine hicut(s,nd,fcut,delt,n,icaus)

!  Butterworth hicut filter order 2*n (n<=8) (see Kanasewich, 
!    Time Sequence Analysis in Geophysics, Third Edition, 
!    University of Alberta Press,1981)
!  written by W. B. Joyner 01/07/97
!  causal if icaus.eq.1 - zero phase shift otherwise
!  s(j) input = the time series to be filtered - output = the 
!    filtered series
!  dimension of s(j) must be at least as large as
!    nd+3.0*float(n)/(fcut*delt)
!  nd = the number of points in the time series
!  fcut = the cutoff frequency
!  delt = the timestep
! Dates: xx/xx/xx - Written by Bill Joyner
!        01/07/00 - D. Boore added check for fcut = 0.0d0, in which case
!                   no filter is applied.  He also cleaned up the
!                   appearance of the code (indented statements in
!                   loops, etc.)
!        09/12/00 - double precision statements added so that the output
!                   series has sufficient precision for double integration.
!        11/08/00 - Increased dimension of s from 50000 to 100000
!        02/14/01 - Modified/corrected real numbers function calls to 
!                   double precision - cds
!        02/22/01 - Removed dimension of s (it is up to the user to specify
!                   it properly)
!        07/20/01 - No filter of fcut .le. 0.0
!        11/18/12 - Increased array dimensions for that n<=16

      real s(*)
      real*8 fact(16),b1(16),b2(16)
!      real*8 fact(8),b1(8),b2(8)
      real*8 pi,w1,xp,yp,x1,x2,y1,y2

      if (fcut <= 0.0) return       ! Added by DMB

      pi=4.0d0*datan(1.0d0)
      w1=2.0d0*pi*fcut
      w1=2.0d0*dtan(w1*delt/2.0d0)/delt

      do k = 1, n
        fact(k)=1.0d0/((2.0d0/(w1*delt))**2+
     :   4.0d0*dsin(pi*dfloat(2*k-1)/dfloat(4*n))/(w1*delt)+1.0d0)
        b1(k)=2.0d0*(1.0d0-(2.0d0/(w1*delt))**2)*fact(k)
        b2(k)=((2.0d0/(w1*delt))**2-
     :   4.0d0*dsin(pi*dfloat(2*k-1)/dfloat(4*n))/(w1*delt)+1.0d0)
     :   *fact(k)
      end do

      np2=nd

      if (icaus /= 1) then
        npad=3.0d0*dfloat(n)/(fcut*delt)
        np1=nd+1
        np2=nd+npad
        do j = np1, np2
          s(j)=0.0
        end do
      end if

      do k = 1, n
        x1=0.0d0
        x2=0.0d0
        y1=0.0d0
        y2=0.0d0
        do j = 1, np2
          xp=s(j)
          yp=fact(k)*(xp+2.0d0*x1+x2)-b1(k)*y1-b2(k)*y2
          s(j)=yp
          y2=y1
          y1=yp
          x2=x1
          x1=xp
        end do
      end do

      if(icaus /= 1) then
        do k = 1, n
          x1=0.0d0
          x2=0.0d0
          y1=0.0d0
          y2=0.0d0
          do j = 1, np2
            xp=s(np2-j+1)
            yp=fact(k)*(xp+2.0d0*x1+x2)-b1(k)*y1-b2(k)*y2
            s(np2-j+1)=yp
            y2=y1
            y1=yp
            x2=x1
            x1=xp
          end do
        end do
      end if

      return
      end
!* --------------------------------------------------------------------- HICUT


!* ---------------------------------------------------------------------- LOCUT
      subroutine locut(s,nd,fcut,delt,nroll,icaus)

c  Butterworth locut filter order 2*nroll (nroll<=8) (see Kanasewich, 
c    Time Sequence Analysis in Geophysics, Third Edition, 
c    University of Alberta Press, 1981)
c  written by W. B. Joyner 01/07/97
c  causal if icaus.eq.1 - zero phase shift otherwise
c  s(j) input = the time series to be filtered - output = the 
c    filtered series
c  dimension of s(j) must be at least as large as 
c    nd+3.0*float(nroll)/(fcut*delt)
c  nd = the number of points in the time series
c  fcut = the cutoff frequency
c  delt = the timestep

* There is a fundamental difference in the shapes of a causal and acausal 
* filter.

* The response for a causal filter is given by eq. 15.8-6 in Kanasewich:
*    Y = sqrt((f/fcut)**(2*n)/(1+(f/fcut)**(2*n))), where n = 2*nroll
* The acausal filter is obtained by applying the causal filter twice.
* The consequence is that the response of the acausal filter is
*    Y =      (f/fcut)**(2*n)/(1+(f/fcut)**(2*n)),  where n = 2*nroll
* Note that for the same nroll as input, for f/fcut << 1
* the causal filter goes as
*    Y -> (f/fcut)**(2*nroll)
* whereas the acausal filter falls off more rapidly:
*    Y -> (f/fcut)**(4*nroll)
* Also note the difference when f = fcut:
*   causal:  Y = 1/sqrt(2)
*  acausal:  Y = 1/2
* This is true no matter what value is chosen for nroll.  Because of this,
* it is not possible to make the response of the causal and acausal filters the
* same.  In particular, caution should be used if nroll is chosen so as to
* make the low-frequency asymptotes the same.  This requires nroll for the 
* acausal filter to be half that of the causal filter.  The problem is that
* the response for frequencies higher than fcut will be reduced for the
* acausal as compared to the causal filter.  


* Dates: xx/xx/xx - Written by Bill Joyner
*        09/30/97 - double precision statements added so that the output
*                   series has sufficient precision for double integration.
*        12/17/99 - D. Boore added check for fcut = 0.0, in which case
*                   no filter is applied.  He also cleaned up the
*                   appearance of the code (indented statements in
*                   loops, etc.)
*        02/04/00 - Changed "n" to "nroll" to eliminate confusion with
*                   Kanesewich, who uses "n" as the order (=2*nroll)
*        09/12/00 - Made trig functions double precision
*        11/08/00 - Increased dimension of s from 50000 to 100000
*        02/14/01 - Modified/corrected real numbers function calls to 
*                   double precision - cds
*        02/15/01 - Added response of acausal filter in the comments
*        02/22/01 - Removed dimension of s (it is up to the user to specify
*                   it properly)
*        02/22/01 - Added more discussion about the shapes of the causal
*                   and acausal filters.
*        07/20/01 - No filter of fcut .le. 0.0
!        08/01/12 - Added "dble" to b1(k)= statement and changed "2.0d0" to "2d0"
!                   (before doing that, part of the statement was highlighted in
!                   red when viewed in Lahey's ED4W, although I don't know why).

      real s(*)
      real*8 fact(8),b1(8),b2(8)
      real*8 pi,w1,xp,yp,x1,x2,y1,y2


      if (fcut .le. 0.0) return       ! Added by DMB

      pi=4.0d0*datan(1.0d0)
      w1=2.0d0*pi*fcut
      w1=2.0d0*dtan(w1*delt/2.0d0)/delt

      do k=1,nroll
        fact(k)=1.0d0/(1.0d0+dsin(pi*dfloat(2*k-1)/dfloat(4*nroll))
     *   *w1*delt+
     *   (w1*delt/2.0d0)**2)
        b1(k)=( (w1*dble(delt))**2/2d0-2d0 )*fact(k)
        b2(k)=(1.0d0-dsin(pi*dfloat(2*k-1)/dfloat(4*nroll))*w1*delt+
     *   (w1*delt/2.0d0)**2)*fact(k)
      end do

      np2=nd

      if(icaus.ne.1) then
        npad=3.0*float(nroll)/(fcut*delt)
        np1=nd+1
        np2=nd+npad
        do j=np1,np2
          s(j)=0.0
        end do
      end if

      do k=1,nroll
        x1=0.0d0
        x2=0.0d0
        y1=0.0d0
        y2=0.0d0
        do j=1,np2
          xp=s(j)
          yp=fact(k)*(xp-2.0d0*x1+x2)-b1(k)*y1-b2(k)*y2
          s(j)=yp
          y2=y1
          y1=yp
          x2=x1
          x1=xp
        end do
      end do

      if(icaus.ne.1) then
        do k=1,nroll
          x1=0.0d0
          x2=0.0d0
          y1=0.0d0
          y2=0.0d0
          do j=1,np2
            xp=s(np2-j+1)
            yp=fact(k)*(xp-2.0d0*x1+x2)-b1(k)*y1-b2(k)*y2
            s(np2-j+1)=yp
            y2=y1
            y1=yp
            x2=x1
            x1=xp
          end do
        end do
      end if

      return
      end
!* ---------------------------------------------------------------------- LOCUT

!* ------------------------ BEGIN NOTCH ------------------------
      subroutine notch(s,nd,fr,delt,d,icaus)
c  notch filter (see Kanasewich, Time Sequence Analysis in 
c    Geophysics, Third Edition, University of Alberta Press, 
c    1981; Shanks, Feb. 1967 Geophysics)
c  written by W. B. Joyner 01/07/97
c  causal if icaus.eq.1 - zero phase shift otherwise
c  s(j) input = the time series to be filtered - output = the 
c    filtered series
c  dimension of s(j) must be at least as large as
c    nd+3.0*pi/d
c  nd = the number of points in the time series
c  fr = the reject frequency
c  delt = the timestep
c  d = parameter << 1 - the smaller d the narrower the notch - 
c    the value 0.01 is suggested for first trial
* Dates: 01/07/97 - Written by W. Joyner
*        07/20/01 - Option to return (no filtering) if fr < 0.0
!        03/20/10 - Tidied up by D. Boore

      real s(*)
      pi=4.0*atan(1.0)
      
      IF (fr < 0.0) THEN
        return
      END IF
      
      r = 1.0+d
      w = pi*fr*delt*2.0
      cosw = cos(w)
      fact = (1.0+2.0*cosw/r+1.0/r**2)/(2.0*(1.0+cosw))
      a1 = -2.0*cosw
      b1 = -2.0*cosw/r
      b2 = 1.0/r**2
      np2 = nd
      
      IF (icaus /= 1) THEN
      
        npad = 3.0*pi/d
        np1 = nd+1
        np2 = nd+npad
        
        DO j = np1, np2
          s(j) = 0.0
        END DO
        
      END IF
      
      x1 =0.0
      x2 =0.0
      y1 =0.0
      y2 =0.0
      
      DO j = 1, np2
      
        xp  = s(j)
        yp  = fact*(xp+a1*x1+x2)-b1*y1-b2*y2
        s(j) = yp
        y2 = y1
        y1 = yp
        x2 = x1
        x1 = xp
        
      END DO
      
      IF (icaus /= 1) THEN
      
        x1 = 0.0
        x2 = 0.0
        y1 = 0.0
        y2 = 0.0
        
        DO j = 1, np2 
        
          xp = s(np2-j+1)
          yp = fact*(xp+a1*x1+x2)-b1*y1-b2*y2
          s(np2-j+1) = yp
          y2 = y1
          y1 = yp
          x2 = x1
          x1 = xp
          
        END DO
        
      END IF
      
      RETURN
      
      END
!* ------------------------ END NOTCH ------------------------

 
